require 'yaml'
require 'chef/role'
require 'chef/environment'

module Knife
  module Undev
    class Converter

      def initialize(filename, klass = 'role')
        @filename = filename
        @klass = klass
        @comment_counter = 0
      end

      def stringify_keys(obj)
        return obj.inject({}){|memo,(k,v)| memo[k.to_s] =  stringify_keys(v); memo} if obj.is_a? Hash
        return obj.inject([]){|memo,v    | memo           << stringify_keys(v); memo} if obj.is_a? Array
        obj
      end

      def to_yml
        case @klass
        when 'env'
          env_to_yml
        when 'role'
          role_to_yml
        end
      end

      def safe_delete(hash, key)
        hash.delete(key) if hash[key].nil? || hash[key].empty?
      end

      def comment(hash, value='')
        hash["#comment_#{@comment_counter+=1}"] = ""
      end

      def role_to_yml
        r = Chef::Role.new
        r.from_file(@filename)
        hash = stringify_keys(r.to_hash)
        result_hash = Hash.new
        hash.delete('chef_type')
        hash.delete('json_class')
        safe_delete(hash, 'override_attributes')
        safe_delete(hash, 'run_list')
        result_hash['name'] = hash.delete('name')
        result_hash['description'] = hash.delete('description')
        comment(result_hash)
        result_hash['env_run_lists'] = hash.delete('env_run_lists')
        comment(result_hash)
        result_hash.merge(hash).to_yaml.each_line do |line|
          puts '' if line.match /#comment_\d+/
          next if line.match /#comment_\d+/
          next if line.match '^---'
          puts line
        end
      end

      def env_to_yml
        r = Chef::Environment.new
        r.from_file(@filename)
        hash = stringify_keys(r.to_hash)
        hash.delete('chef_type')
        hash.delete('json_class')
        safe_delete(hash, 'override_attributes')
        safe_delete(hash, 'default_attributes')
        safe_delete(hash, 'cookbook_versions')
        safe_delete(hash, 'description')
        hash.to_yaml.each_line do |line|
          next if line.match '^---'
          puts line
        end
      end

    end
  end
end
